/* BdfServer_API - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.api.storage;

import java.io.IOException;
import java.io.InputStream;
import net.fichotheque.EditOrigin;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.mimetype.MimeTypeResolver;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public interface ResourceStorage {

    public final static String VAR_NAME = "var";
    public final static String ETC_NAME = "etc";
    public final static String DIST_NAME = "dist";

    public String getName();

    public boolean containsResource(RelativePath path);

    public DocStream getResourceDocStream(RelativePath path, MimeTypeResolver mimeTypeResolver);

    public void saveResource(RelativePath path, InputStream inputStream, EditOrigin editOrigin) throws IOException;

    public boolean removeResource(RelativePath path, EditOrigin editOrigin);

    public boolean createFolder(RelativePath path);

    public ResourceFolder getRoot();

    @Nullable
    public default ResourceFolder getResourceFolder(RelativePath path) {
        String[] pathArray = path.toArray();
        ResourceFolder currentFolder = getRoot();
        int length = pathArray.length;
        int depth = 0;
        boolean exists = true;
        while (exists) {
            exists = false;
            for (ResourceFolder subfolder : currentFolder.getSubfolderList()) {
                if (subfolder.getName().equals(pathArray[depth])) {
                    exists = true;
                    currentFolder = subfolder;
                    depth++;
                    break;
                }
            }
            if (depth == length) {
                return currentFolder;
            }
        }
        return null;
    }

}
