/* BdfServer_Commands - Copyright (c) 2012-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.administration;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.roles.Role;
import fr.exemole.bdfserver.api.roles.RoleEditor;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.text.ParseException;
import net.fichotheque.ExistingIdException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class RoleCreationCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "RoleCreation";
    public final static String COMMANDKEY = "_ ROL-01";
    public final static String NEWROLE_PARAMNAME = "newrole";
    private String newRoleName;

    public RoleCreationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        try {
            BdfServerEditor bdfServerEditor = bdfServer.getBdfServerEditor();
            RoleEditor roleEditor = bdfServerEditor.getRoleEditor();
            Role role = roleEditor.createRole(newRoleName);
            bdfServerEditor.saveChanges();
            setDone("_ done.administration.rolecreation", newRoleName);
            putResultObject(ROLE_OBJ, role);
        } catch (ParseException pe) {
            throw new ShouldNotOccurException("Test done before");
        } catch (ExistingIdException eie) {
            throw new ShouldNotOccurException("Test done before");
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        BdfInstructionUtils.checkGlobalAdminPermission(bdfServer, bdfUser);
        newRoleName = getMandatory(NEWROLE_PARAMNAME);
        newRoleName = newRoleName.trim();
        if (newRoleName.length() == 0) {
            setError("_ error.empty.rolename", newRoleName);
            return false;
        }
        if (!StringUtils.isTechnicalName(newRoleName, true)) {
            setError("_ error.wrong.rolename", newRoleName);
            return false;
        }
        Role role = bdfServer.getPermissionManager().getRole(newRoleName);
        if (role != null) {
            setError("_ error.existing.role", newRoleName);
            return false;
        }
        return true;
    }

}
