/* BdfServer_Commands - Copyright (c) 2012-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.administration;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import net.fichotheque.EditOrigin;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public class StreamTextChangeCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "StreamTextChange";
    public final static String PATH_PARAMNAME = "path";
    public final static String CONTENT_PARAMNAME = "content";
    private RelativePath relativePath;
    private String text;

    public StreamTextChangeCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        try {
            boolean done = saveBdfResourceStreamText();
            if (done) {
                setDone("_ done.administration.streamtextchange", relativePath.toString());
                BdfServerUtils.checkResourceChange(bdfServer, relativePath);
            }
        } catch (IOException ioe) {
            setError("_ error.exception.streamtext_io", ioe.getMessage());
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        BdfInstructionUtils.checkGlobalAdminPermission(bdfServer, bdfUser);
        String path = getMandatory(PATH_PARAMNAME);
        try {
            relativePath = RelativePath.parse(path);
        } catch (ParseException pe) {
            setError("_ error.wrong.resourcepath", path);
            return false;
        }
        if (!StorageUtils.isValidResourceRelativePath(relativePath)) {
            setError("_ error.wrong.resourcepath", path);
            return false;
        }
        text = getMandatory(CONTENT_PARAMNAME);
        return true;
    }

    private boolean saveBdfResourceStreamText() throws IOException {
        EditOrigin editOrigin = bdfUser.newEditOrigin(Domains.ADMINISTRATION + "/" + COMMANDNAME);
        try (InputStream is = IOUtils.toInputStream(text, "UTF-8")) {
            StorageUtils.saveResource(bdfServer, relativePath, is, editOrigin);
        }
        return true;
    }

}
