/* BdfServer_Commands - Copyright (c) 2015-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.importation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.interaction.TableParserParametersParser;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.request.FileValue;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.tableparser.TableParser;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractImportParseCommand extends AbstractBdfCommand {

    public final static String FIELDSORIGIN_PARAMNAME = "fieldsorigin";
    public final static String INPUT_FIELDSORIGIN_PARAMVALUE = "input";
    public final static String FIRSTLINE_FIELDSORIGIN_PARAMVALUE = "firstline";
    public final static String FIELDS_PARAMNAME = "fields";
    public final static String CONTENT_PARAMNAME = "content";
    public final static String FILE_PARAMNAME = "file";
    public final static String CHARSET_PARAMNAME = "charset";
    protected String[] fieldsArray;
    protected String[][] lines;

    public AbstractImportParseCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);

    }

    protected boolean checkLines() throws BdfInstructionException {
        String fieldsOrigin = getMandatory(FIELDSORIGIN_PARAMNAME);
        String contentString = getContent();
        if (contentString == null) {
            return false;
        }
        TableParser.Parameters tableParserParameters = TableParserParametersParser.parse(requestMap);
        this.lines = TableParser.parse(contentString, tableParserParameters);
        if (fieldsOrigin.equals(INPUT_FIELDSORIGIN_PARAMVALUE)) {
            this.fieldsArray = StringUtils.getTechnicalTokens(getMandatory(FIELDS_PARAMNAME), true);
        } else {
            int length = lines.length;
            if (length == 0) {
                this.fieldsArray = new String[0];
            } else {
                this.fieldsArray = lines[0];
                String[][] tempo = new String[length - 1][];
                System.arraycopy(lines, 1, tempo, 0, length - 1);
                this.lines = tempo;
            }
        }

        return true;
    }

    public String getContent() throws BdfInstructionException {
        FileValue fileValue = requestMap.getFileValue(FILE_PARAMNAME);
        if (fileValue == null) {
            return getMandatory(CONTENT_PARAMNAME);
        }
        String charset = requestMap.getParameter(CHARSET_PARAMNAME);
        if (charset != null) {
            charset = charset.trim();
            if (charset.length() > 0) {
                boolean supported;
                try {
                    supported = Charset.isSupported(charset);
                } catch (IllegalCharsetNameException icne) {
                    supported = false;
                }
                if (!supported) {
                    throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(CHARSET_PARAMNAME, charset));
                }
            } else {
                charset = null;
            }
        }
        if (charset == null) {
            charset = "UTF-8";
        }
        String contentString;
        try (InputStream is = fileValue.getInputStream()) {
            contentString = IOUtils.toString(is, charset);
        } catch (IOException ioe) {
            contentString = null;
            setError("_ error.exception.io", ioe.getMessage());
        }
        fileValue.free();
        return contentString;
    }

}
