/* BdfServer_HtmlProducers - Copyright (c) 2015-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.addenda;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.domains.AddendaDomain;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.jslib.CommonJsLibs;
import net.fichotheque.addenda.Addenda;
import net.mapeadores.util.html.CommandBox;


/**
 *
 * @author Vincent Calame
 */
public class AddendaMetadataFormHtmlProducer extends BdfServerHtmlProducer {

    private final PermissionSummary permissionSummary;
    private final Addenda addenda;

    public AddendaMetadataFormHtmlProducer(ProducerParameters producerParameters, Addenda addenda) {
        this(producerParameters.getBdfServer(), producerParameters.getBdfUser(), producerParameters.getPermissionSummary(), addenda);
    }

    public AddendaMetadataFormHtmlProducer(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, Addenda addenda) {
        super(bdfServer, bdfUser);
        addJsLib(CommonJsLibs.COMMANDTEST);
        addJsLib(CommonJsLibs.SUBSETCHANGE);
        addThemeCss("addenda.css");
        this.permissionSummary = permissionSummary;
        this.addenda = addenda;
    }

    @Override
    public void printHtml() {
        start();
        BdfHtmlUtils.printSubsetHeader(this, bdfServer, bdfUser, permissionSummary, addenda, AddendaDomain.ADDENDA_METADATAFORM_PAGE);
        AddendaHtmlUtils.printAddendaToolbar(this, AddendaDomain.ADDENDA_METADATAFORM_PAGE, addenda);
        printCommandMessageUnit();
        CommandBox commandBox = CommandBox.init()
                .action(Domains.ADDENDA)
                .family("ADD")
                .page(AddendaDomain.ADDENDA_METADATAFORM_PAGE);
        AddendaCommandBoxUtils.printAddendaPhrasesBox(this, commandBox, addenda, bdfServer);
        end();
    }

}
