/* BdfServer_HtmlProducers - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.corpus;

import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.domains.CorpusDomain;
import fr.exemole.bdfserver.api.ui.CommentUi;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.commands.corpus.FieldOptionsCommand;
import fr.exemole.bdfserver.commands.corpus.FieldRemoveCommand;
import fr.exemole.bdfserver.html.BdfHtmlConstants;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.consumers.Button;
import fr.exemole.bdfserver.html.consumers.Grid;
import fr.exemole.bdfserver.html.consumers.LangRows;
import fr.exemole.bdfserver.tools.BH;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import java.util.Locale;
import net.fichotheque.Subset;
import net.fichotheque.SubsetItem;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.Langs;
import net.mapeadores.util.text.Labels;


/**
 *
 * @author Vincent Calame
 */
public final class CorpusHtmlUtils implements CorpusDomain {

    private CorpusHtmlUtils() {
    }

    public static boolean printCorpusToolbar(HtmlPrinter hp, Corpus corpus, String currentPage) {
        hp
                .NAV("subset-Toolbar")
                .__(printCommands(hp, corpus, currentPage))
                .__(printInclude(hp, corpus, currentPage))
                ._NAV();
        return true;
    }

    private static boolean printCommands(HtmlPrinter hp, Corpus corpus, String currentPage) {
        boolean withRemoveField = (CorpusMetadataUtils.containsRemoveableField(corpus.getCorpusMetadata()));
        hp
                .DIV("subset-Buttons")
                .SPAN("subset-SmallLabel")
                .__localize("_ title.global.commands")
                .__colon()
                ._SPAN()
                .__(printLink(hp, "action-Labels", corpus, CorpusDomain.CORPUS_PHRASESFORM_PAGE, "_ link.corpus.corpusphrasesform", currentPage))
                .__(printLink(hp, "action-FieldCreate", corpus, CorpusDomain.FIELD_CREATIONFORM_PAGE, "_ link.corpus.fieldcreationform", currentPage))
                .__(printLink(hp, "action-Options", corpus, CorpusDomain.FIELD_OPTIONSFORM_PAGE, FieldOptionsCommand.COMMANDKEY, currentPage))
                .__(CorpusHtmlUtils.printLink(withRemoveField, hp, "action-FieldRemove", corpus, CorpusDomain.FIELD_REMOVEFORM_PAGE, FieldRemoveCommand.COMMANDKEY, currentPage))
                .__(printLink(hp, "action-ComponentPosition", corpus, CorpusDomain.UI_COMPONENTPOSITIONFORM_PAGE, "_ link.corpus.uicomponentpositionform", currentPage))
                .__(printLink(hp, "action-ComponentOptions", corpus, CorpusDomain.UI_COMPONENTOPTIONSFORM_PAGE, "_ link.corpus.uicomponentoptionsform", currentPage))
                .__(printLink(hp, "action-ComponentAttributes", corpus, CorpusDomain.UI_COMPONENTATTRIBUTESFORM_PAGE, "_ link.corpus.uicomponentattributesform", currentPage))
                .__(printLink(hp, "action-Advanced", corpus, CorpusDomain.CORPUS_ADVANCEDCOMMANDS_PAGE, "_ link.corpus.corpusadvancedcommands", currentPage))
                .__(printLink(hp, "action-Diagrams", corpus, CorpusDomain.CORPUS_DIAGRAM_PAGE, "_ link.corpus.diagram", currentPage))
                .__(printLink(hp, "action-History", corpus, CorpusDomain.REMOVEDLIST_PAGE, "_ link.corpus.removedlist", currentPage))
                .__(printRefresh(hp, corpus))
                ._DIV();
        return true;
    }

    private static boolean printInclude(HtmlPrinter hp, Corpus corpus, String currentPage) {
        hp
                .DIV("subset-Buttons")
                .SPAN("subset-SmallLabel")
                .__localize("_ title.corpus.include")
                .__colon()
                ._SPAN()
                .__(printLink(hp, "action-Special", corpus, CorpusDomain.INCLUDE_SPECIALFORM_PAGE, "_ link.corpus.corpusinclude_specialform", currentPage))
                .__(printLink(hp, "action-Thesaurus", corpus, CorpusDomain.INCLUDE_THESAURUSFORM_PAGE, "_ link.corpus.corpusinclude_thesaurusform", currentPage))
                .__(printLink(hp, "action-Corpus", corpus, CorpusDomain.INCLUDE_CORPUSFORM_PAGE, "_ link.corpus.corpusinclude_corpusform", currentPage))
                .__(printLink(hp, "action-Album", corpus, CorpusDomain.INCLUDE_ALBUMFORM_PAGE, "_ link.corpus.corpusinclude_albumform", currentPage))
                .__(printLink(hp, "action-Addenda", corpus, CorpusDomain.INCLUDE_ADDENDAFORM_PAGE, "_ link.corpus.corpusinclude_addendaform", currentPage))
                .__(printLink(hp, "action-Comment", corpus, CorpusDomain.INCLUDE_COMMENTFORM_PAGE, "_ link.corpus.corpusinclude_commentform", currentPage))
                ._DIV();
        return true;
    }

    private static boolean printLink(boolean test, HtmlPrinter hp, String action, Corpus corpus, String page, String titleLocKey, String currentPage) {
        if (!test) {
            return false;
        }
        return printLink(hp, action, corpus, page, titleLocKey, currentPage);
    }

    private static boolean printLink(HtmlPrinter hp, String action, Corpus corpus, String page, String titleLocKey, String currentPage) {
        if (page.equals(currentPage)) {
            hp
                    .SPAN("global-button-Transparent global-button-Current " + action)
                    .__(Button.ICON)
                    ._SPAN();
        } else {
            hp
                    .A(HA
                            .href(BH.domain(Domains.CORPUS).subset(corpus).page(page))
                            .titleLocKey(titleLocKey)
                            .classes("global-button-Transparent " + action))
                    .__(Button.ICON)
                    ._A();
        }
        return true;
    }

    public static boolean printFicheHeader(HtmlPrinter hp, FicheMeta ficheMeta, Lang workingLang, Locale formatLocale) {
        SubsetItem masterItem = null;
        Subset masterSubset = ficheMeta.getCorpus().getMasterSubset();
        if (masterSubset != null) {
            masterItem = masterSubset.getSubsetItemById(ficheMeta.getId());
        }
        String title = CorpusMetadataUtils.getFicheTitle(ficheMeta, workingLang, formatLocale);
        hp
                .HEADER("subset-ItemHeader")
                .__(BdfHtmlUtils.printFicheTitle(hp, title, ficheMeta.isDiscarded(), masterItem, workingLang, formatLocale))
                ._HEADER();
        return true;
    }

    public static boolean printFicheHeader(HtmlPrinter hp, String title, boolean discarded, SubsetItem masterItem, Lang workingLang, Locale formatLocale) {
        hp
                .HEADER("subset-ItemHeader")
                .__(BdfHtmlUtils.printFicheTitle(hp, title, discarded, masterItem, workingLang, formatLocale))
                ._HEADER();
        return true;
    }

    public static boolean printIncludeChangeDetails(HtmlPrinter hp, CommandBox commandBox, Corpus corpus, IncludeUi includeUi, Langs workingLangs, boolean withJavascript) {
        Labels labels = includeUi.getCustomLabels();
        if (labels == null) {
            hp
                    .P("global-SubTitle")
                    .__localize("_ info.corpus.defaultlabelusage")
                    ._P();
        } else {
            hp
                    .__(Grid.START)
                    .__(LangRows.init("", labels, workingLangs).displayOnly(true))
                    .__(Grid.END);
        }
        hp
                .DETAILS("command-Details")
                .SUMMARY()
                .__localize("_ link.global.change")
                ._SUMMARY()
                .__(CorpusCommandBoxUtils.printIncludeChangeBox(hp, commandBox, corpus, includeUi, workingLangs))
                ._DETAILS();
        return true;
    }

    public static boolean printIncludeRemoveDetails(HtmlPrinter hp, CommandBox commandBox, Corpus corpus, IncludeUi includeUi) {
        hp
                .DETAILS("command-Details")
                .SUMMARY()
                .__localize("_ link.global.delete")
                ._SUMMARY()
                .__(CorpusCommandBoxUtils.printIncludeRemoveBox(hp, commandBox, corpus, includeUi))
                ._DETAILS();
        return true;
    }

    public static boolean printCommentChangeDetails(HtmlPrinter hp, CommandBox commandBox, Corpus corpus, CommentUi commentUi, Lang[] workingLangArray) {
        int position = commentUi.getLocation();
        String[] htmlArray = UiUtils.toStringArray(commentUi, workingLangArray);
        hp
                .P("global-SubTitle")
                .__localize("_ label.corpus.commentlocation")
                .__colon()
                .__(() -> {
                    boolean suivant = false;
                    if ((position & CommentUi.FORM_BITVALUE) != 0) {
                        suivant = true;
                        hp
                                .__localize("_ label.corpus.commentlocation_form");
                    }
                    if ((position & CommentUi.TEMPLATE_BITVALUE) != 0) {
                        if (suivant) {
                            hp
                                    .__escape(", ");
                        }
                        hp
                                .__localize("_ label.corpus.commentlocation_template");
                    }
                })
                ._P()
                .__(Grid.START)
                .__(LangRows.init("", workingLangArray, htmlArray).displayOnly(true).preformatted(true))
                .__(Grid.END)
                .DETAILS("command-Details")
                .SUMMARY()
                .__localize("_ link.global.change")
                ._SUMMARY()
                .__(CorpusCommandBoxUtils.printCommentChangeBox(hp, commandBox, corpus, commentUi, workingLangArray))
                ._DETAILS();
        return true;
    }

    public static boolean printCommentRemoveDetails(HtmlPrinter hp, CommandBox commandBox, Corpus corpus, CommentUi commentUi) {
        hp
                .DETAILS("command-Details")
                .SUMMARY()
                .__localize("_ link.global.delete")
                ._SUMMARY()
                .__(CorpusCommandBoxUtils.printCommentRemoveBox(hp, commandBox, corpus, commentUi))
                ._DETAILS();
        return true;
    }

    public static String getFicheItemMessageKey(short ficheItemType) {
        switch (ficheItemType) {
            case CorpusField.PERSONNE_FIELD:
                return "_ label.corpus.ficheitemtype_personne";
            case CorpusField.LANGUE_FIELD:
                return "_ label.corpus.ficheitemtype_langue";
            case CorpusField.DATATION_FIELD:
                return "_ label.corpus.ficheitemtype_datation";
            case CorpusField.PAYS_FIELD:
                return "_ label.corpus.ficheitemtype_pays";
            case CorpusField.COURRIEL_FIELD:
                return "_ label.corpus.ficheitemtype_courriel";
            case CorpusField.LINK_FIELD:
                return "_ label.corpus.ficheitemtype_link";
            case CorpusField.ITEM_FIELD:
                return "_ label.corpus.ficheitemtype_item";
            case CorpusField.NOMBRE_FIELD:
                return "_ label.corpus.ficheitemtype_nombre";
            case CorpusField.MONTANT_FIELD:
                return "_ label.corpus.ficheitemtype_montant";
            case CorpusField.GEOPOINT_FIELD:
                return "_ label.corpus.ficheitemtype_geopoint";
            case CorpusField.PARA_FIELD:
                return "_ label.corpus.ficheitemtype_para";
            case CorpusField.IMAGE_FIELD:
                return "_ label.corpus.ficheitemtype_image";
            default:
                throw new SwitchException("Unknown type: " + ficheItemType);
        }
    }

    private static boolean printRefresh(HtmlPrinter hp, Corpus corpus) {
        hp
                .A(HA
                        .href(BH.domain(Domains.CORPUS).page(CorpusDomain.CORPUS_PAGE).subset(corpus))
                        .titleLocKey("_ link.global.reload")
                        .classes("global-button-Transparent action-Refresh")
                        .target(BdfHtmlConstants.LIST_FRAME))
                .__(Button.ICON)
                ._A();
        return true;
    }

}
