/* BdfServer_HtmlProducers - Copyright (c) 2015-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.CorpusDomain;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.ui.SubsetIncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.consumers.PageUnit;
import fr.exemole.bdfserver.html.jslib.CommonJsLibs;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import java.util.List;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.mapeadores.util.html.CommandBox;


/**
 *
 * @author Vincent Calame
 */
public class IncludeAddendaFormHtmlProducer extends BdfServerHtmlProducer {

    private final PermissionSummary permissionSummary;
    private final Corpus corpus;

    public IncludeAddendaFormHtmlProducer(ProducerParameters producerParameters, Corpus corpus) {
        this(producerParameters.getBdfServer(), producerParameters.getBdfUser(), producerParameters.getPermissionSummary(), corpus);
    }

    public IncludeAddendaFormHtmlProducer(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, Corpus corpus) {
        super(bdfServer, bdfUser);
        addJsLib(CommonJsLibs.COMMANDTEST);
        addJsLib(CommonJsLibs.DEPLOY);
        addJsLib(CommonJsLibs.SUBSETCHANGE);
        addJsLib(CommonJsLibs.CODEMIRRORMODE);
        addThemeCss("corpus.css");
        this.permissionSummary = permissionSummary;
        this.corpus = corpus;
    }

    @Override
    public void printHtml() {
        UiComponents uiComponents = bdfServer.getUiManager().getMainUiComponents(corpus);
        List<SubsetIncludeUi> list = UiUtils.getSubsetIncludeUiList(uiComponents, SubsetKey.CATEGORY_ADDENDA);
        start();
        BdfHtmlUtils.printSubsetHeader(this, bdfServer, bdfUser, permissionSummary, corpus, CorpusDomain.INCLUDE_ADDENDAFORM_PAGE);
        CorpusHtmlUtils.printCorpusToolbar(this, corpus, CorpusDomain.INCLUDE_ADDENDAFORM_PAGE);
        printCommandMessageUnit();
        CommandBox commandBox = CommandBox.init()
                .action(Domains.CORPUS)
                .family("CRP")
                .page(CorpusDomain.INCLUDE_ADDENDAFORM_PAGE)
                .mode(InteractionConstants.SUBUNIT_MODE);
        this
                .__(PageUnit.start("action-Addenda", "_ title.corpus.addendainclude_list"));
        if (list.isEmpty()) {
            this
                    .P()
                    .__localize("_ info.corpus.empty_addendainclude")
                    ._P();
        } else {
            this
                    .DL("global-DL");
            for (SubsetIncludeUi includeUi : list) {
                this
                        .DT()
                        .__(BdfHtmlUtils.printCodeMirrorSpan(this, includeUi, bdfServer, workingLang))
                        ._DT()
                        .DD()
                        .__(CorpusHtmlUtils.printIncludeChangeDetails(this, commandBox, corpus, includeUi, bdfServer.getLangConfiguration().getWorkingLangs(), isWithJavascript()))
                        .__(CorpusHtmlUtils.printIncludeRemoveDetails(this, commandBox, corpus, includeUi))
                        ._DD();
            }
            this
                    ._DL();
        }
        this
                .__(PageUnit.END);
        CorpusCommandBoxUtils.printSubsetIncludeCreationBox(this, commandBox.mode(""), corpus, bdfServer, workingLang, SubsetKey.CATEGORY_ADDENDA);
        end();
    }

}
