/* BdfServer_HtmlProducers - Copyright (c) 2012-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.thesaurus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.ThesaurusDomain;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.jslib.CommonJsLibs;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.ThesaurusUtils;
import net.mapeadores.util.html.CommandBox;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusAdvancedCommandsHtmlProducer extends BdfServerHtmlProducer {

    private final PermissionSummary permissionSummary;
    private final Thesaurus thesaurus;

    public ThesaurusAdvancedCommandsHtmlProducer(ProducerParameters producerParameters, Thesaurus thesaurus) {
        this(producerParameters.getBdfServer(), producerParameters.getBdfUser(), producerParameters.getPermissionSummary(), thesaurus);
    }

    public ThesaurusAdvancedCommandsHtmlProducer(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, Thesaurus thesaurus) {
        super(bdfServer, bdfUser);
        addJsLib(CommonJsLibs.COMMANDTEST);
        addJsLib(CommonJsLibs.DEPLOY);
        addJsLib(CommonJsLibs.SUBSETCHANGE);
        addJsLib(CommonJsLibs.CODEMIRRORMODE);
        addThemeCss("thesaurus.css");
        this.permissionSummary = permissionSummary;
        this.thesaurus = thesaurus;
    }

    @Override
    public void printHtml() {
        start();
        BdfHtmlUtils.printSubsetHeader(this, bdfServer, bdfUser, permissionSummary, thesaurus, ThesaurusDomain.THESAURUS_ADVANCEDCOMMANDS_PAGE);
        ThesaurusHtmlUtils.printThesaurusToolbar(this, ThesaurusDomain.THESAURUS_ADVANCEDCOMMANDS_PAGE, thesaurus);
        printCommandMessageUnit();
        CommandBox commandBox = CommandBox.init()
                .action(Domains.THESAURUS)
                .family("THS");
        if ((permissionSummary.isGlobalAdmin()) && (FichothequeUtils.isRemoveable(thesaurus))) {
            commandBox
                    .errorPage(ThesaurusDomain.THESAURUS_ADVANCEDCOMMANDS_PAGE)
                    .page(InteractionConstants.MESSAGE_PAGE);
            ThesaurusCommandBoxUtils.printThesaurusRemoveBox(this, commandBox, thesaurus);
        }
        commandBox
                .errorPage(null)
                .page(ThesaurusDomain.THESAURUS_ADVANCEDCOMMANDS_PAGE);
        ThesaurusCommandBoxUtils.printThesaurusAttributeChangeBox(this, commandBox, thesaurus);
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            ThesaurusCommandBoxUtils.printIdalphaSortBox(this, commandBox, thesaurus, null);
        }
        ThesaurusCommandBoxUtils.printChildrenReorderBox(this, commandBox, thesaurus, null);
        if (permissionSummary.isGlobalAdmin()) {
            commandBox
                    .page(ThesaurusDomain.THESAURUS_METADATAFORM_PAGE)
                    .errorPage(ThesaurusDomain.THESAURUS_ADVANCEDCOMMANDS_PAGE);
            ThesaurusCommandBoxUtils.printThesaurusCloneBox(this, commandBox, thesaurus, workingLang);
        }
        ThesaurusCommandBoxUtils.printThesaurusCleaningBox(this, commandBox, thesaurus, workingLang);
        end();
    }

}
