/* global BDF,$$,ID,Transformation,CodeMirror */

Transformation.ContentForm = {};

Transformation.ContentForm.load = function (templateContent, isInStock) {
    if (!isInStock) {
        isInStock = false;
    }
    var templateKey = templateContent.templateKey;
    var contentPath = templateContent.path;
    var formId = BDF.generateId();
    var mandatory = false;
    var testFunction = null;
    _checkContentPath();
    var saveId = formId + "_button_save";
    var textareaId = formId + "_content";
    var htmlContent = BDF.render("transformation:contentform", {
        action: BDF.URL + "exportation",
        formId: formId,
        templateKey: templateKey,
        split: Transformation.split(templateKey),
        path: contentPath,
        content: templateContent.content,
        title: _getTitle(),
        withTestButton: (testFunction !== null),
        withDeleteButton: !mandatory
    });
    if (isInStock) {
        BDF.Pane.addStockUnit(htmlContent);
    } else {
        $$("layout_main").html(htmlContent);
    }
    $$(formId, "button_delete").click(function () {
        Transformation.Overlay.showTemplateContentRemoveConfirm(templateKey, contentPath);
    });
    if (testFunction) {
        $$(formId, "button_test").click(testFunction);
    }
    $$(formId, "button_stock").click(function () {
        BDF.Pane.moveMainToStock();
    });
    $$(formId).ajaxForm({
        beforeSerialize: function ($form, options) {
            if (isInStock) {
                return false;
            } else {
                codeMirror.save();
                return true;
            }
        },
        dataType: "json",
        success: function (data, textStatus, jqXHR, $form) {
            BDF.Pane.setChange(false);
            _saveCallback(data);
        }
    });
    var codeMirror = BDF.toCodeMirror(ID(textareaId), {
        lineWrapping: false,
        lineNumbers: true,
        indentUnit: 4,
        theme: 'bdf',
        mode: BDF.getMatchingCodeMirrorMode(contentPath),
        extraKeys: {
            "Ctrl-S": function(cm) {
                if (!$$(saveId).prop("disabled")) {
                    $$(saveId).click();
                }
            }
        }
    });
    codeMirror.on("change",function () {
        BDF.Pane.setChange(true);
    });
    $$(formId).data("stockHandler", _stockHandler);
    BDF.Pane.checkState();

    
    function _checkContentPath() {
        switch(contentPath) {
            case "transformer.xsl":
                mandatory = true;
                if (Transformation.getTestPath(templateKey)) {
                    testFunction = function () {
                        Transformation.Overlay.showSimpleTemplateTest(templateKey);
                    };
                }
                break;
            case "content.xsl":
                mandatory = true;
                if (Transformation.getTestPath(templateKey)) {
                    testFunction = function () {
                        var testPath = Transformation.getTestPath(templateKey);
                        if (Transformation.isCompilation(templateKey)) {
                            testPath += "?limit=1";
                        }
                        window.location.href = testPath;
                    };
                }
                break;
            case "extraction.xml":
                testFunction = function () {
                    Transformation.Overlay.showExtractionTest(templateKey);
                };
                break;
        }
    }
    
    function _getTitle() {
        let templateDescription = Transformation.getTemplateDescription(templateKey);
        if (templateDescription && templateDescription.title) {
            return templateDescription.title;
        } else {
            return "";
        }
    }
    
    function _saveCallback(data) {
        if (BDF.isError(data)) {
            return;
        }
        BDF.showCommandMessage(data.commandMessage);
        Transformation.updateTemplateDescription(data.templateDescription);
    }
    
    function _stockHandler(stock) {
        isInStock = stock;
        codeMirror.setOption("readOnly", stock);
    }
    
};
