/* FichothequeLib_Tools - Copyright (c) 2009-2018 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction.builders;

import java.util.ArrayList;
import java.util.List;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.extraction.def.ExtraitAddendaDef;
import net.fichotheque.extraction.def.ExtraitAlbumDef;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.ExtraitThesaurusDef;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.MotcleRattachementDef;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.mapeadores.util.attr.Attributes;


/**
 *
 * @author Vincent Calame
 */
public class FilterUnitListBuilder {

    private final List<FilterUnit> list = new ArrayList<FilterUnit>();

    public FilterUnitListBuilder() {
    }

    public FilterUnitListBuilder addFieldKeyFilter(FieldKey fieldKey, Attributes attributes) {
        list.add(FilterUnitUtils.toFieldKeyFilterUnit(fieldKey, attributes));
        return this;
    }

    public FilterUnitListBuilder addFieldNameFilter(short category, String fieldName, Attributes attributes) {
        if (fieldName.endsWith("*")) {
            int length = fieldName.length();
            if (length == 1) {
                return this;
            }
            String prefix = fieldName.substring(0, length - 1);
            list.add(FilterUnitUtils.toFieldNamePrefixFilterUnit(category, prefix, attributes));
        } else {
            try {
                FieldKey fieldKey = FieldKey.parse(category, fieldName);
                addFieldKeyFilter(fieldKey, attributes);
            } catch (java.text.ParseException pe) {
            }
        }
        return this;
    }

    public FilterUnitListBuilder addLotFilter(String tagName, List<FilterUnit> filterUnitList) {
        if (tagName != null) {
            tagName = tagName.trim();
            if (tagName.length() == 0) {
                tagName = null;
            }
        }
        list.add(FilterUnitUtils.toLotFilterUnit(tagName, filterUnitList));
        return this;
    }

    public FilterUnitListBuilder addEnteteFilter() {
        list.add(FilterUnitUtils.ENTETE_FILTERUNIT);
        return this;
    }

    public FilterUnitListBuilder addCorpsdeficheFilter() {
        list.add(FilterUnitUtils.CORPSDEFICHE_FILTERUNIT);
        return this;
    }

    public FilterUnitListBuilder addChronoFilter() {
        list.add(FilterUnitUtils.CHRONO_FILTERUNIT);
        return this;
    }

    public FilterUnitListBuilder addIntituleFilter() {
        list.add(FilterUnitUtils.INTITULE_FILTERUNIT);
        return this;
    }

    public FilterUnitListBuilder addExtraitCorpusFilter(ExtraitCorpusDef extraitCorpusDef) {
        list.add(FilterUnitUtils.toExtraitCorpusFilterUnit(extraitCorpusDef));
        return this;
    }

    public FilterUnitListBuilder addExtraitThesaurusFilter(ExtraitThesaurusDef extraitThesaurusDef) {
        list.add(FilterUnitUtils.toExtraitThesaurusFilterUnit(extraitThesaurusDef));
        return this;
    }

    public FilterUnitListBuilder addExtraitAlbumFilter(ExtraitAlbumDef extraitAlbumDef) {
        list.add(FilterUnitUtils.toExtraitAlbumFilterUnit(extraitAlbumDef));
        return this;
    }

    public FilterUnitListBuilder addExtraitAddendaFilter(ExtraitAddendaDef extraitAddendaDef) {
        list.add(FilterUnitUtils.toExtraitAddendaFilterUnit(extraitAddendaDef));
        return this;
    }

    public FilterUnitListBuilder addFicheRattachementFilter(FicheRattachementDef ficheRattachementDef) {
        list.add(FilterUnitUtils.toFicheRattachementFilterUnit(ficheRattachementDef));
        return this;
    }

    public FilterUnitListBuilder addMotcleRattachementFilter(MotcleRattachementDef motcleRattachementDef) {
        list.add(FilterUnitUtils.toMotcleRattachementFilterUnit(motcleRattachementDef));
        return this;
    }

    public List<FilterUnit> toFilterUnitList() {
        return FilterUnitUtils.toImmutableList(list);
    }

    public final static FilterUnitListBuilder init() {
        return new FilterUnitListBuilder();
    }

}
