/* FichothequeLib_Tools - Copyright (c) 2013-2018 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.selection;

import java.util.LinkedHashSet;
import java.util.Set;
import net.fichotheque.selection.CroisementCondition;
import net.mapeadores.util.primitives.Ranges;


/**
 *
 * @author Vincent Calame
 */
public class CroisementConditionBuilder {

    private final Set<String> modeSet = new LinkedHashSet<String>();
    private Ranges poidsRanges = null;
    private boolean isExclusion = false;

    public CroisementConditionBuilder() {
    }

    /**
     * Accepte la valeur _default qui est convertie en chaine vide.
     */
    public CroisementConditionBuilder addLienMode(String mode) {
        if (mode.equals("_default")) {
            mode = "";
        }
        modeSet.add(mode);
        return this;
    }

    public CroisementConditionBuilder setPoidsRanges(Ranges poidsRanges, boolean isExclusion) {
        this.poidsRanges = poidsRanges;
        this.isExclusion = isExclusion;
        return this;
    }

    public boolean isEmpty() {
        return ((poidsRanges == null) && (modeSet.isEmpty()));
    }

    public CroisementCondition toCroisementCondition() {
        String[] lienModeArray = null;
        int size = modeSet.size();
        if (size > 0) {
            lienModeArray = modeSet.toArray(new String[size]);
        }
        return new InternalCroisementCondition(lienModeArray, poidsRanges, isExclusion);
    }

    public static CroisementConditionBuilder init() {
        return new CroisementConditionBuilder();
    }


    private static class InternalCroisementCondition implements CroisementCondition {

        private final String[] lienModeArray;
        private final Ranges poidsRanges;
        private final boolean isExclusion;

        private InternalCroisementCondition(String[] lienModeArray, Ranges poidsRanges, boolean isExclusion) {
            this.lienModeArray = lienModeArray;
            this.poidsRanges = poidsRanges;
            this.isExclusion = isExclusion;
        }

        @Override
        public String[] getLienModeArray() {
            return lienModeArray;
        }

        @Override
        public Ranges getPoidsRanges() {
            return poidsRanges;
        }

        @Override
        public boolean isExclusionRangeList() {
            return isExclusion;
        }

    }

}
