/* FichothequeLib_Tools - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.selection;

import net.fichotheque.SubsetKey;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionsConstants;
import net.mapeadores.util.conditions.ConditionsUtils;
import net.mapeadores.util.instruction.Argument;
import net.mapeadores.util.instruction.Instruction;
import net.mapeadores.util.instruction.InstructionParser;


/**
 *
 * @author Vincent Calame
 */
public final class MotcleQueryParser {

    private MotcleQueryParser() {

    }

    public static boolean isMotcleQueryInstruction(Instruction instruction) {
        for (Argument argument : instruction) {
            String key = argument.getKey();
            if (key.equals("thesaurus")) {
                return true;
            }
            if (key.equals("motcle-q")) {
                return true;
            }
            if (key.equals("motcle-q-a")) {
                return true;
            }
        }
        return false;
    }

    public static MotcleQuery parseInstruction(Instruction instruction) {
        MotcleQueryBuilder motcleQueryBuilder = new MotcleQueryBuilder();
        Condition condition = null;
        int length = instruction.size();
        for (int i = 0; i < length; i++) {
            Argument argument = instruction.get(i);
            String key = argument.getKey();
            String value = argument.getValue();
            if (key.equals("thesaurus")) {
                if (value == null) {
                    continue;
                }
                motcleQueryBuilder.addThesaurus(FichothequeUtils.toSubsetKeySet(SubsetKey.CATEGORY_THESAURUS, value));
            } else if (key.equals("motcle-q")) {
                if (value != null) {
                    condition = ConditionsUtils.parseCondition(value, ConditionsConstants.MODE_OR);
                }
            } else if (key.equals("motcle-q-a")) {
                if (value != null) {
                    condition = ConditionsUtils.parseCondition(value, ConditionsConstants.MODE_AND);
                }
            } else if (key.equals("rattachement")) {
                if ((value == null) || (!value.equals("0"))) {
                    motcleQueryBuilder.setIncludeSatellites(true);
                }
            }
        }
        motcleQueryBuilder.setContentCondition(condition, MotcleQuery.ONLY_IDALPHA_SCOPE);
        return motcleQueryBuilder.toMotcleQuery();
    }

    public static MotcleQuery parse(String s) {
        Instruction instruction = InstructionParser.parse(s, null);
        if (instruction == null) {
            return null;
        }
        return parseInstruction(instruction);
    }

}
