/* UtilLib - Copyright (c) 2016-2018 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.localisation.Message;


/**
 *
 * @author Vincent Calame
 */
public class CategorizedMessageBuffer {

    private final List<InternalCategorizedMessageEntry> entryList = new ArrayList<InternalCategorizedMessageEntry>();

    public CategorizedMessageBuffer() {

    }

    public boolean isEmpty() {
        return (entryList.isEmpty());
    }

    public void clear() {
        entryList.clear();
    }

    public void add(String category, Message message) {
        if (category == null) {
            throw new IllegalArgumentException("category is null");
        }
        if (message == null) {
            throw new IllegalArgumentException("message is null");
        }
        entryList.add(new InternalCategorizedMessageEntry(category, message));
    }

    public List<CategorizedMessageEntry> toEntryList() {
        return LogUtils.wrap(entryList.toArray(new CategorizedMessageEntry[entryList.size()]));
    }

    public List<Message> toMessageList(String category) {
        List<Message> list = new ArrayList<Message>();
        for (InternalCategorizedMessageEntry entry : entryList) {
            if (entry.getCategory().startsWith(category)) {
                list.add(entry.getMessage());
            }
        }
        return list;
    }


    private static class InternalCategorizedMessageEntry implements CategorizedMessageEntry {

        private final String category;
        private final Message message;

        private InternalCategorizedMessageEntry(String category, Message message) {
            this.category = category;
            this.message = message;
        }

        @Override
        public String getCategory() {
            return category;
        }

        @Override
        public Message getMessage() {
            return message;
        }

    }

}
